@@ -29,13 +29,13 @@ import (
 
 const (
 	APPLICATION_NAME        = "Zabbix Agent"
-	ZABBIX_REVDATE          = "22 February 2021"
+	ZABBIX_REVDATE          = "26 April 2021"
 	ZABBIX_VERSION_MAJOR    = 5
 	ZABBIX_VERSION_MINOR    = 0
-	ZABBIX_VERSION_PATCH    = 9
+	ZABBIX_VERSION_PATCH    = 11
 	ZABBIX_VERSION_RC       = ""
 	ZABBIX_VERSION_RC_NUM   = "2400"
-	ZABBIX_VERSION_REVISION = "4d07aaafe2"
+	ZABBIX_VERSION_REVISION = "15ae5548ce"
 	copyrightMessage        = "Copyright (C) 2021 Zabbix SIA\n" +
 		"License GPLv2+: GNU GPL version 2 or later <http://gnu.org/licenses/gpl.html>.\n" +
 		"This is free software: you are free to change and redistribute it according to\n" +
