@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2020 Zabbix SIA
+** Copyright (C) 2001-2021 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -34,9 +34,6 @@ int	NET_DNS_RECORD(AGENT_REQUEST *request, AGENT_RESULT *result);
 int	PROC_MEM(AGENT_REQUEST *request, AGENT_RESULT *result);
 int	PROC_NUM(AGENT_REQUEST *request, AGENT_RESULT *result);
 int	SYSTEM_BOOTTIME(AGENT_REQUEST *request, AGENT_RESULT *result);
-int	WEB_PAGE_GET(AGENT_REQUEST *request, AGENT_RESULT *result);
-int	WEB_PAGE_PERF(AGENT_REQUEST *request, AGENT_RESULT *result);
-int	WEB_PAGE_REGEXP(AGENT_REQUEST *request, AGENT_RESULT *result);
 int	NET_TCP_LISTEN(AGENT_REQUEST *request, AGENT_RESULT *result);
 int	CHECK_SERVICE(AGENT_REQUEST *request, AGENT_RESULT *result);
 int	CHECK_SERVICE_PERF(AGENT_REQUEST *request, AGENT_RESULT *result);
@@ -77,24 +74,12 @@ func resolveMetric(key string) (cfunc unsafe.Pointer) {
 		cfunc = unsafe.Pointer(C.NET_DNS)
 	case "net.dns.record":
 		cfunc = unsafe.Pointer(C.NET_DNS_RECORD)
-	case "proc.mem":
-		cfunc = unsafe.Pointer(C.PROC_MEM)
 	case "proc.num":
 		cfunc = unsafe.Pointer(C.PROC_NUM)
 	case "system.boottime":
 		cfunc = unsafe.Pointer(C.SYSTEM_BOOTTIME)
-	case "web.page.get":
-		cfunc = unsafe.Pointer(C.WEB_PAGE_GET)
-	case "web.page.perf":
-		cfunc = unsafe.Pointer(C.WEB_PAGE_PERF)
-	case "web.page.regexp":
-		cfunc = unsafe.Pointer(C.WEB_PAGE_REGEXP)
 	case "net.tcp.listen":
 		cfunc = unsafe.Pointer(C.NET_TCP_LISTEN)
-	case "net.tcp.service", "net.udp.service":
-		cfunc = unsafe.Pointer(C.CHECK_SERVICE)
-	case "net.tcp.service.perf", "net.udp.service.perf":
-		cfunc = unsafe.Pointer(C.CHECK_SERVICE_PERF)
 	case "net.udp.listen":
 		cfunc = unsafe.Pointer(C.NET_UDP_LISTEN)
 	case "sensor":
@@ -115,30 +100,14 @@ func resolveMetric(key string) (cfunc unsafe.Pointer) {
 		cfunc = unsafe.Pointer(C.SYSTEM_HW_MACADDR)
 	case "system.sw.os":
 		cfunc = unsafe.Pointer(C.SYSTEM_SW_OS)
-	case "system.sw.packages":
-		cfunc = unsafe.Pointer(C.SYSTEM_SW_PACKAGES)
 	case "system.swap.in":
 		cfunc = unsafe.Pointer(C.SYSTEM_SWAP_IN)
 	case "system.swap.out":
 		cfunc = unsafe.Pointer(C.SYSTEM_SWAP_OUT)
-	case "system.swap.size":
-		cfunc = unsafe.Pointer(C.SYSTEM_SWAP_SIZE)
-	case "system.users.num":
-		cfunc = unsafe.Pointer(C.SYSTEM_USERS_NUM)
 	case "vfs.dir.count":
 		cfunc = unsafe.Pointer(C.VFS_DIR_COUNT)
 	case "vfs.dir.size":
 		cfunc = unsafe.Pointer(C.VFS_DIR_SIZE)
-	case "vfs.fs.discovery":
-		cfunc = unsafe.Pointer(C.VFS_FS_DISCOVERY)
-	case "vfs.fs.inode":
-		cfunc = unsafe.Pointer(C.VFS_FS_INODE)
-	case "vfs.fs.size":
-		cfunc = unsafe.Pointer(C.VFS_FS_SIZE)
-	case "vfs.fs.get":
-		cfunc = unsafe.Pointer(C.VFS_FS_GET)
-	case "vm.memory.size":
-		cfunc = unsafe.Pointer(C.VM_MEMORY_SIZE)
 	}
 	return
 }
