@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2020 Zabbix SIA
+** Copyright (C) 2001-2021 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -34,19 +34,38 @@ static zbx_mutex_t	proxy_lock = ZBX_MUTEX_NULL;
 #define	LOCK_PROXY_HISTORY	if (0 != (program_type & ZBX_PROGRAM_TYPE_PROXY_PASSIVE)) zbx_mutex_lock(proxy_lock)
 #define	UNLOCK_PROXY_HISTORY	if (0 != (program_type & ZBX_PROGRAM_TYPE_PROXY_PASSIVE)) zbx_mutex_unlock(proxy_lock)
 
-int	zbx_send_proxy_data_response(const DC_PROXY *proxy, zbx_socket_t *sock, const char *info)
+int	zbx_send_proxy_data_response(const DC_PROXY *proxy, zbx_socket_t *sock, const char *info, int upload_status)
 {
 	struct zbx_json		json;
 	zbx_vector_ptr_t	tasks;
-	int			ret, flags = ZBX_TCP_PROTOCOL;
+	int			ret, flags = ZBX_TCP_PROTOCOL, status;
 
 	zbx_vector_ptr_create(&tasks);
 
-	zbx_tm_get_remote_tasks(&tasks, proxy->hostid);
-
 	zbx_json_init(&json, ZBX_JSON_STAT_BUF_LEN);
 
-	zbx_json_addstring(&json, ZBX_PROTO_TAG_RESPONSE, ZBX_PROTO_VALUE_SUCCESS, ZBX_JSON_TYPE_STRING);
+	switch (upload_status)
+	{
+		case ZBX_PROXY_UPLOAD_DISABLED:
+			zbx_json_addstring(&json, ZBX_PROTO_TAG_PROXY_UPLOAD, ZBX_PROTO_VALUE_PROXY_UPLOAD_DISABLED,
+					ZBX_JSON_TYPE_STRING);
+			status = FAIL;
+			break;
+		case ZBX_PROXY_UPLOAD_ENABLED:
+			zbx_json_addstring(&json, ZBX_PROTO_TAG_PROXY_UPLOAD, ZBX_PROTO_VALUE_PROXY_UPLOAD_ENABLED,
+					ZBX_JSON_TYPE_STRING);
+			ZBX_FALLTHROUGH;
+		default:
+			status = SUCCEED;
+	}
+
+	if (SUCCEED == status)
+	{
+		zbx_json_addstring(&json, ZBX_PROTO_TAG_RESPONSE, ZBX_PROTO_VALUE_SUCCESS, ZBX_JSON_TYPE_STRING);
+		zbx_tm_get_remote_tasks(&tasks, proxy->hostid);
+	}
+	else
+		zbx_json_addstring(&json, ZBX_PROTO_TAG_RESPONSE, ZBX_PROTO_VALUE_FAILED, ZBX_JSON_TYPE_STRING);
 
 	if (NULL != info && '\0' != *info)
 		zbx_json_addstring(&json, ZBX_PROTO_TAG_INFO, info, ZBX_JSON_TYPE_STRING);
@@ -84,9 +103,9 @@ int	zbx_send_proxy_data_response(const DC_PROXY *proxy, zbx_socket_t *sock, cons
  ******************************************************************************/
 void	zbx_recv_proxy_data(zbx_socket_t *sock, struct zbx_json_parse *jp, zbx_timespec_t *ts)
 {
-	int		ret = FAIL, status;
-	char		*error = NULL;
-	DC_PROXY	proxy;
+	int			ret = FAIL, upload_status = 0, status, version, responded = 0;
+	char			*error = NULL;
+	DC_PROXY		proxy;
 
 	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __func__);
 
@@ -104,20 +123,27 @@ void	zbx_recv_proxy_data(zbx_socket_t *sock, struct zbx_json_parse *jp, zbx_time
 		goto out;
 	}
 
-	zbx_update_proxy_data(&proxy, zbx_get_proxy_protocol_version(jp), time(NULL),
-			(0 != (sock->protocol & ZBX_TCP_COMPRESS) ? 1 : 0));
+	version = zbx_get_proxy_protocol_version(jp);
 
-	if (SUCCEED != zbx_check_protocol_version(&proxy))
+	if (SUCCEED != zbx_check_protocol_version(&proxy, version))
 	{
 		goto out;
 	}
 
-	if (SUCCEED != (ret = process_proxy_data(&proxy, jp, ts, &error)))
+	if (FAIL == (ret = zbx_hc_check_proxy(proxy.hostid)))
 	{
-		zabbix_log(LOG_LEVEL_WARNING, "received invalid proxy data from proxy \"%s\" at \"%s\": %s",
-				proxy.host, sock->peer, error);
-		status = FAIL;
-		goto out;
+		upload_status = ZBX_PROXY_UPLOAD_DISABLED;
+	}
+	else
+	{
+		upload_status = ZBX_PROXY_UPLOAD_ENABLED;
+
+		if (SUCCEED != (ret = process_proxy_data(&proxy, jp, ts, HOST_STATUS_PROXY_ACTIVE, NULL, &error)))
+		{
+			zabbix_log(LOG_LEVEL_WARNING, "received invalid proxy data from proxy \"%s\" at \"%s\": %s",
+					proxy.host, sock->peer, error);
+			goto out;
+		}
 	}
 
 	if (!ZBX_IS_RUNNING())
@@ -125,21 +151,29 @@ void	zbx_recv_proxy_data(zbx_socket_t *sock, struct zbx_json_parse *jp, zbx_time
 		error = zbx_strdup(error, "Zabbix server shutdown in progress");
 		zabbix_log(LOG_LEVEL_WARNING, "cannot process proxy data from active proxy at \"%s\": %s",
 				sock->peer, error);
-		ret = status = FAIL;
+		ret = FAIL;
 		goto out;
 	}
-	else
-		zbx_send_proxy_data_response(&proxy, sock, error);
+
+	zbx_send_proxy_data_response(&proxy, sock, error, upload_status);
+	responded = 1;
 
 out:
-	if (FAIL == ret)
+	if (SUCCEED == status)	/* moved the unpredictable long operation to the end */
+				/* we are trying to save info about lastaccess to detect communication problem */
+	{
+		zbx_update_proxy_data(&proxy, version, ts->sec,
+				(0 != (sock->protocol & ZBX_TCP_COMPRESS) ? 1 : 0), 0);
+	}
+
+	if (0 == responded)
 	{
 		int	flags = ZBX_TCP_PROTOCOL;
 
 		if (0 != (sock->protocol & ZBX_TCP_COMPRESS))
 			flags |= ZBX_TCP_COMPRESS;
 
-		zbx_send_response_ext(sock, status, error, NULL, flags, CONFIG_TIMEOUT);
+		zbx_send_response_ext(sock, ret, error, NULL, flags, CONFIG_TIMEOUT);
 	}
 
 	zbx_free(error);
@@ -187,7 +221,7 @@ void	zbx_send_proxy_data(zbx_socket_t *sock, zbx_timespec_t *ts)
 	struct zbx_json		j;
 	zbx_uint64_t		areg_lastid = 0, history_lastid = 0, discovery_lastid = 0;
 	char			*error = NULL;
-	int			availability_ts, more_history, more_discovery, more_areg;
+	int			availability_ts, more_history, more_discovery, more_areg, proxy_delay;
 	zbx_vector_ptr_t	tasks;
 	struct zbx_json_parse	jp, jp_tasks;
 
@@ -224,6 +258,9 @@ void	zbx_send_proxy_data(zbx_socket_t *sock, zbx_timespec_t *ts)
 	zbx_json_adduint64(&j, ZBX_PROTO_TAG_CLOCK, ts->sec);
 	zbx_json_adduint64(&j, ZBX_PROTO_TAG_NS, ts->ns);
 
+	if (0 != history_lastid && 0 != (proxy_delay = proxy_get_delay(history_lastid)))
+		zbx_json_adduint64(&j, ZBX_PROTO_TAG_PROXY_DELAY, proxy_delay);
+
 	if (SUCCEED == send_data_to_server(sock, j.buffer, &error))
 	{
 		zbx_set_availability_diff_ts(availability_ts);
